#include "StdAfx.h"
#include "registry.h"

CRegistry::CRegistry(HKEY hRoot, LPCTSTR lpszSubKey)
{
	m_hKey = NULL;
	char	ClassStr[1024];
	DWORD	d = REG_CREATED_NEW_KEY;

	if (ERROR_SUCCESS != ::RegCreateKeyEx(hRoot, lpszSubKey, 0, ClassStr, REG_OPTION_NON_VOLATILE, KEY_READ|KEY_WRITE, NULL, &m_hKey, &d))	
		m_hKey = NULL;
}

CRegistry::~CRegistry(void)
{
	if (m_hKey != NULL)
	{
		::RegCloseKey(m_hKey);
	}
}

BOOL CRegistry::GetValueStr(HKEY hRoot, LPCTSTR lpszSubKey, LPCTSTR lpszName, char* Result)
{
	BOOL bSuccess = FALSE;

	HKEY	hKey;
	DWORD	dwType, dwSize = 1024;
	char	szBuf[1024];
	char*	str = NULL;

	if(ERROR_SUCCESS == RegOpenKeyEx(hRoot, lpszSubKey, 0, KEY_QUERY_VALUE, &hKey))
	{
		if(ERROR_SUCCESS == RegQueryValueEx(hKey, lpszName, NULL, &dwType, (LPBYTE)(szBuf), &dwSize))
		{
			if( ( REG_SZ == dwType || REG_MULTI_SZ == dwType || REG_EXPAND_SZ == dwType) && dwSize >= 0)
			{
				szBuf[dwSize] = '\0';
				strcpy(Result, szBuf);
				bSuccess = TRUE;
			}
		}
		RegCloseKey(hKey);
	}

	return bSuccess;
}

BOOL CRegistry::SetValueStr(HKEY hRoot, LPCTSTR lpszSubKey, LPCTSTR lpszName, LPCTSTR lpszValue)
{
	HKEY	hKey;
	char	ClassStr[1024];
	DWORD	d = REG_CREATED_NEW_KEY;

	if(ERROR_SUCCESS != RegCreateKeyEx(hRoot, lpszSubKey, 0, ClassStr, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &d))	
		return FALSE;

	if(ERROR_SUCCESS != RegSetValueEx(hKey, lpszName, 0, REG_SZ, (BYTE*)(lpszValue), lstrlen(lpszValue)))
	{
		RegCloseKey(hKey);
		return FALSE;
	}

	RegCloseKey(hKey);

	return TRUE;
}

BOOL CRegistry::GetValueDWORD(HKEY hRoot, LPCTSTR lpszSubKey, LPCTSTR lpszName, DWORD *Result)
{
	BOOL bSuccess = FALSE;

	HKEY	hKey;
	DWORD	keyType=REG_DWORD, n = sizeof(DWORD);
	DWORD 	buf = 0;

	if(ERROR_SUCCESS == RegOpenKeyEx(hRoot, lpszSubKey, 0, KEY_QUERY_VALUE, &hKey))
	{
		if(ERROR_SUCCESS == RegQueryValueEx(hKey, lpszName, NULL, &keyType, (LPBYTE)&buf, &n))
		{
			*Result = (DWORD)(buf);
		}
		else
		{
			RegCloseKey(hKey);
			return FALSE;
		}
		RegCloseKey(hKey);
	} 
	else
	{ 
		return FALSE;
	}

	return TRUE;

}

BOOL CRegistry::SetValueDWORD(HKEY hRoot, LPCTSTR lpszSubKey, LPCTSTR lpszName, DWORD Value)
{
	HKEY	hKey;
	char	ClassStr[1024];
	DWORD	d = REG_CREATED_NEW_KEY;

	if(ERROR_SUCCESS != RegCreateKeyEx(hRoot, lpszSubKey, 0, ClassStr, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hKey, &d))	
		return FALSE;

	d = (DWORD)Value;
	if(ERROR_SUCCESS != RegSetValueEx(hKey, lpszName, 0, REG_DWORD, (BYTE*)(&d), sizeof(DWORD)))
	{
		RegCloseKey(hKey);
		return FALSE;
	}

	RegCloseKey(hKey);

	return TRUE;
}

BOOL CRegistry::DeleteValue(HKEY hRoot, LPCTSTR lpszSubKey, LPCTSTR lpszName)
{
	BOOL bSuccess = FALSE;

	HKEY	hKey;
	BOOL ret;

	if(ERROR_SUCCESS == RegOpenKeyEx(hRoot, lpszSubKey, 0, KEY_SET_VALUE, &hKey))
	{	
		ret = RegDeleteValue(hKey, lpszName);				
		RegCloseKey(hKey);

		bSuccess = ret;
	}
	else
	{
		bSuccess = FALSE;
	}

	return bSuccess;
}

BOOL CRegistry::DeleteKey(HKEY hRoot, LPCTSTR lpszSubKey)
{
	BOOL bSuccess = FALSE;

	if (IsKeyExist(hRoot, lpszSubKey))
	{
		if (ERROR_SUCCESS == ::RegDeleteKey(hRoot, lpszSubKey))
			bSuccess = TRUE;
	}

	return bSuccess;
}

BOOL CRegistry::IsKeyExist(HKEY hRoot, LPCTSTR lpszSubKey)
{
	BOOL bSuccess = FALSE;
	HKEY hKey;

	if(ERROR_SUCCESS == RegOpenKeyEx(hRoot, lpszSubKey, 0, KEY_QUERY_VALUE, &hKey))
	{
		RegCloseKey(hKey);
		bSuccess = TRUE;
	}
	else 
		bSuccess = FALSE;

	return bSuccess;
}

BOOL CRegistry::SetValue(LPCTSTR lpszName, DWORD dwType, const BYTE* lpData, DWORD dwSize)
{
	ATLASSERT(m_hKey != NULL);

	LONG lResult = ::RegSetValueEx(m_hKey, lpszName, 0, dwType, lpData, dwSize);
	if (lResult != ERROR_SUCCESS) 
		return FALSE;

	return TRUE;
}

BOOL CRegistry::GetValue(LPCTSTR lpszName, LPDWORD lpdwType, LPBYTE lpData, LPDWORD lpdwSize)
{
	ATLASSERT(m_hKey != NULL);

	LONG lResult = ::RegQueryValueEx(m_hKey, lpszName, 0, lpdwType, lpData, lpdwSize); 
	if (lResult != ERROR_SUCCESS) 
		return FALSE;

	return TRUE;
}

BOOL CRegistry::DeleteValue(LPCTSTR lpszName)
{
	ATLASSERT(m_hKey != NULL);

	LONG lResult = RegDeleteValue(m_hKey, lpszName);
	if (lResult != ERROR_SUCCESS)
		return FALSE;

	return TRUE;
}

// data reading functions
int	 CRegistry::ReadInt(CString strName, int nDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_DWORD;
	int n;
	DWORD dwSize = sizeof(n);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL,	&dwType, (LPBYTE)&n, &dwSize); 
	if ( lResult != ERROR_SUCCESS ) 
	{
		n = nDefault;
	}
	
	return n;
}

BOOL CRegistry::ReadBool(CString strName, BOOL bDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_DWORD;
	BOOL b;
	DWORD dwSize = sizeof(b);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL,	&dwType, (LPBYTE)&b, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		b = bDefault;
	}

	return b;
}

BOOL CRegistry::ReadFont(CString strName, LOGFONT* pLogfont)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	DWORD dwType = REG_BINARY;
	DWORD dwSize = sizeof(LOGFONT);
	LOGFONT lf;

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)&lf, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}

	if (bSuccess)
	{
		*pLogfont = lf;
	}

	return bSuccess;
}

BOOL CRegistry::ReadPoint(CString strName, CPoint* pPoint)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	DWORD dwType = REG_BINARY;
	DWORD dwSize = sizeof(CPoint);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)pPoint, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}

	return bSuccess;
}

BOOL CRegistry::ReadSize(CString strName, CSize* pSize)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	DWORD dwType = REG_BINARY;
	DWORD dwSize = sizeof(CSize);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)pSize, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}

	return bSuccess;
}

BOOL CRegistry::ReadRect(CString strName, CRect* pRect)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	DWORD dwType = REG_BINARY;
	DWORD dwSize = sizeof(CRect);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)pRect, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}

	return bSuccess;
}

double	CRegistry::ReadFloat(CString strName, double fDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_DWORD;
	double d;
	DWORD dwSize = sizeof(d);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)&d, &dwSize);
	if ( lResult!= ERROR_SUCCESS) 
	{
		d = fDefault;
	}

	return d;
}

DWORD	CRegistry::ReadDword(CString strName, DWORD dwDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_DWORD;
	DWORD dw;
	DWORD dwSize = sizeof(dw);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)&dw, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		dw = dwDefault;
	}
	
	return dw;
}

CString CRegistry::ReadString(CString strName, CString strDefault)
{
	ATLASSERT(m_hKey != NULL);

	CString strResult = strDefault;

	DWORD dwType = 0;
	DWORD dwSize = 255;
	_TCHAR sz[255];

	ZeroMemory(sz, 255);
	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)sz, &dwSize);
	if (lResult == ERROR_SUCCESS) 
	{
		if (dwType == REG_SZ || dwType == REG_EXPAND_SZ)
		{
			strResult = (LPCTSTR)sz;
		}
	}

	return strResult;
}

COLORREF CRegistry::ReadColor(CString strName, COLORREF rgbDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_BINARY;
	COLORREF rgb;
	DWORD dwSize = sizeof(rgb);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)&rgb, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		rgb = rgbDefault;
	}

	return rgb;
}

time_t CRegistry::ReadDateTime(CString strName, time_t dtDefault)
{
	ATLASSERT(m_hKey != NULL);

	DWORD dwType = REG_BINARY;
	time_t dt;
	DWORD dwSize = sizeof(dt);

	LONG lResult = ::RegQueryValueEx(m_hKey, LPCTSTR(strName), NULL, &dwType, (LPBYTE)&dt, &dwSize);
	if ( lResult != ERROR_SUCCESS) 
	{
		dt = dtDefault;
	}

	return dt;
}


// data writing functions
BOOL CRegistry::WriteInt(CString strName, int nValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_DWORD, (LPBYTE)&nValue, sizeof(nValue));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}

	return bSuccess;
}

BOOL CRegistry::WriteBool(CString strName, BOOL bValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_DWORD, (LPBYTE)&bValue, sizeof(bValue));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
	
	return bSuccess;
}

BOOL CRegistry::WriteFont(CString strName, LOGFONT* pFont)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	LOGFONT lf = *pFont;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)&lf, sizeof(lf));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
	
	return bSuccess;
}

BOOL CRegistry::WritePoint(CString strName, CPoint* pPoint)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)pPoint, sizeof(CPoint));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteSize(CString strName, CSize* pSize)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)pSize, sizeof(CSize));
	if (lResult != ERROR_SUCCESS)
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteRect(CString strName, CRect* pRect)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)pRect, sizeof(CRect));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteFloat(CString strName, double fValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_DWORD, (LPBYTE)&fValue, sizeof(fValue));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteDword(CString strName, DWORD dwValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(dwValue));
	if (lResult != ERROR_SUCCESS)
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteString(CString strName, CString strValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	_TCHAR sz[255];
	
	if (strValue.GetLength() > 254) return FALSE;
	
#ifdef _UNICODE
	wstrcpy(sz, LPCTSTR(strValue));

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_SZ, (LPBYTE)sz, wstrlen(sz) + 1);
	if (lResult != ERROR_SUCCESS)
	{
		bSuccess = FALSE;
	}
#else
	strcpy(sz, LPCTSTR(strValue));

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_SZ, (LPBYTE)sz, lstrlen(sz) + 1);
	if (lResult != ERROR_SUCCESS)
	{
		bSuccess = FALSE;
	}
#endif
		
	return bSuccess;
}

BOOL CRegistry::WriteExpandString(CString strName, CString strValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	_TCHAR sz[255];
	
	if (strValue.GetLength() > 254) return FALSE;
	
#ifdef _UNICODE
	wstrcpy(sz, LPCTSTR(strValue));

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_EXPAND_SZ, (LPBYTE)sz, wstrlen(sz) + 1);
	if (lResult != ERROR_SUCCESS)
	{
		bSuccess = FALSE;
	}
#else
	strcpy(sz, LPCTSTR(strValue));

	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_EXPAND_SZ, (LPBYTE)sz, lstrlen(sz) + 1);
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
#endif
		
	return bSuccess;
}

BOOL CRegistry::WriteColor(CString strName, COLORREF rgbValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)&rgbValue, sizeof(rgbValue));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::WriteDateTime(CString strName, time_t dtValue)
{
	ATLASSERT(m_hKey != NULL);

	BOOL bSuccess = TRUE;
	
	LONG lResult = ::RegSetValueEx(m_hKey, LPCTSTR(strName), 0, REG_BINARY, (LPBYTE)&dtValue, sizeof(dtValue));
	if (lResult != ERROR_SUCCESS) 
	{
		bSuccess = FALSE;
	}
		
	return bSuccess;
}

BOOL CRegistry::GetEnumCount(LPDWORD lpcSubKeyCount, LPDWORD lpcValueCount)
{
	ATLASSERT(m_hKey != NULL);

	LONG lResult = RegQueryInfoKey(m_hKey, NULL, NULL, NULL, lpcSubKeyCount, NULL, NULL, lpcValueCount, NULL, NULL, NULL, NULL);

	if (lResult != ERROR_SUCCESS)
		return FALSE;
	
	return TRUE;
}

BOOL CRegistry::GetEnumKey(DWORD dwIndex, CString& strKeyName)
{
	ATLASSERT(m_hKey != NULL);

	char szKeyName[MAX_PATH];
	DWORD dwCount = MAX_PATH;
	LONG lResult = RegEnumKey(m_hKey, dwIndex, szKeyName, dwCount);
	strKeyName = szKeyName;

	if (lResult != ERROR_SUCCESS)
		return FALSE;

	return TRUE;
}

BOOL CRegistry::GetEnumValue(DWORD dwIndex, CString& strValueName, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
	ATLASSERT(m_hKey != NULL);

	char szValueName[MAX_PATH];
	DWORD dwCount = MAX_PATH;
	LONG lResult = RegEnumValue(m_hKey, dwIndex, szValueName, &dwCount, NULL, lpType, lpData, lpcbData);
	szValueName[dwCount] = '\0';
	strValueName = szValueName;

	if (lResult != ERROR_SUCCESS)
	{
		return FALSE;
	}

	return TRUE;
}
